package bk;

import animation.AlgorithmStage;
import animation.PseudoCodeNode;
import graph.LayeredGraphNode;

/**
 * The stage where the for extremal layouts are computed.
 * @author kolja
 *
 */
public class ExtremalLayoutCalc implements AlgorithmStage {

	public enum LayoutType{
		TOP_BOTTOM_LEFT,
		TOP_BOTTOM_RIGHT,
		BOTTOM_TOP_LEFT,
		BOTTOM_TOP_RIGHT,
		COMBINED
	}
	
	private enum LayoutState
	{
		BLOCK_CALCULATION,
		COMPACTION
	}
	
	private PseudoCodeNode pseudoCode;
	private BlockCalc bc;
	private Compaction cp;
	private LayoutState status;
	private PseudoCodeNode bcNode;
	private PseudoCodeNode cpNode;
	private LayoutType type;
	
	
	public ExtremalLayoutCalc( LayoutType typ, LayeredGraphNode graph )
	{
	    type = typ;
		status = LayoutState.BLOCK_CALCULATION;
		bc = new BlockCalc( graph, typ );
		cp = new Compaction( graph, typ );
	}
	
	@Override
	public StageStatus forwardStep() {
		if( status == LayoutState.BLOCK_CALCULATION )
		{
		    bcNode.setSelected( true );
			if( bc.forwardStep() == StageStatus.FINISHED )
			{
	            bcNode.setSelected( false );
	            cpNode.setSelected( true );
				status = LayoutState.COMPACTION;
			}
			return StageStatus.UNFINISHED;
		}
		if( status == LayoutState.COMPACTION )
		{
			if( cp.forwardStep() == StageStatus.FINISHED )
			{
			    cpNode.setSelected( false );
			    return StageStatus.FINISHED;
			}
		}
		return StageStatus.UNFINISHED;
	}

	@Override
	public StageStatus backwardStep() {
		if( status == LayoutState.BLOCK_CALCULATION )
		{
            bcNode.setSelected( true );
			if( bc.backwardStep() == StageStatus.FINISHED )
			{
	            bcNode.setSelected( false );
                return StageStatus.FINISHED;
			}
		}
		if( status == LayoutState.COMPACTION )
		{
            cpNode.setSelected( true );
			if( cp.backwardStep() == StageStatus.FINISHED )
			{
	            cpNode.setSelected( false );
				status = LayoutState.BLOCK_CALCULATION;
			}
		}
		return StageStatus.UNFINISHED;
	}

    @Override
    public PseudoCodeNode createPseudocodeTree() {
        if( type == LayoutType.TOP_BOTTOM_LEFT )
            pseudoCode = new PseudoCodeNode( "Extremal layout: leftmost upper" );
        if( type == LayoutType.TOP_BOTTOM_RIGHT )
            pseudoCode = new PseudoCodeNode( "Extremal layout: rightmost upper'" );
        if( type == LayoutType.BOTTOM_TOP_LEFT )
            pseudoCode = new PseudoCodeNode( "Extremal layout: leftmost lower'" );;
        if( type == LayoutType.BOTTOM_TOP_RIGHT )
            pseudoCode = new PseudoCodeNode( "Extremal layout: rightmost lower'" );
        bcNode = bc.createPseudocodeTree();
        cpNode = cp.createPseudocodeTree();
        pseudoCode.add( bcNode );
        pseudoCode.add( cpNode );
        return pseudoCode;
    }
}
